<?php

/**
 * ======================================================================
 * LICENSE: This file is subject to the terms and conditions defined in *
 * file 'license.txt', which is part of this source code package.       *
 * ======================================================================
 */

/**
 * Backend posts & pages manager
 * 
 * @package AAM
 * @author Vasyl Martyniuk <vasyl@vasyltech.com>
 */
class AAM_Backend_Feature_Post extends AAM_Backend_Feature_Abstract {
    
    /**
     * Get list for the table
     * 
     * @return string
     * 
     * @access public
     */
    public function getTable() {
        $type = trim(AAM_Core_Request::request('type'));

        if (empty($type)) {
            $response = $this->retrieveTypeList();
        } else {
            $response = $this->retrieveTypeContent($type);
        }

        return $this->wrapTable($response);
    }
    
    /**
     * Retrieve list of registered post types
     * 
     * @return array
     * 
     * @access protected
     */
    protected function retrieveTypeList() {
        $response = array('data' => array());

        foreach (get_post_types(array(), 'objects') as $type) {
            if ($type->public) {
                $response['data'][] = array(
                    $type->name,
                    null,
                    'type',
                    $type->labels->name,
                    'drilldown,manage'
                );
            }
        }

        return $response;
    }

    /**
     * Get post type children
     * 
     * Retrieve list of all posts and terms that belong to specified post type
     * 
     * @param string $type
     * 
     * @return array
     * 
     * @access protected
     */
    protected function retrieveTypeContent($type) {
        $list = array();

        //first retrieve all hierarchical terms that belong to Post Type
        foreach (get_object_taxonomies($type, 'objects') as $tax) {
            if (is_taxonomy_hierarchical($tax->name)) {
                //get all terms that have no parent category
                $list = array_merge($list, $this->retrieveTermList($tax->name));
            }
        }

        //retrieve all posts
        $posts = get_posts(array(
            'post_type'   => $type, 'category' => 0, 
            'numberposts' => -1, 'post_status' => 'any'
        ));

        foreach ($posts as $post) {
            $list[] = array(
                $post->ID,
                get_edit_post_link($post->ID, 'link'),
                'post',
                $post->post_title,
                'manage,edit'
            );
        }

        return array('data' => $list);
    }

    /**
     * Retrieve term list
     * 
     * @param string $taxonomy
     * 
     * @return array
     * 
     * @access protected
     */
    protected function retrieveTermList($taxonomy) {
        $response = array();

        $terms = get_terms($taxonomy, array('hide_empty' => false));

        foreach ($terms as $term) {
            $response[] = array(
                $term->term_id . '|' . $taxonomy,
                get_edit_term_link($term->term_id, $taxonomy),
                'term',
                $term->name,
                'manage,edit'
            );
        }

        return $response;
    }

    /**
     * Prepare response
     * 
     * @param array $response
     * 
     * @return string
     * 
     * @access protected
     */
    protected function wrapTable($response) {
        $response['draw'] = AAM_Core_Request::request('draw');

        return json_encode($response);
    }

    /**
     * Get Post or Term access
     *
     * @return string
     *
     * @access public
     */
    public function getAccess() {
        $type = trim(AAM_Core_Request::post('type'));
        $id   = AAM_Core_Request::post('id');

        $object = AAM_Backend_View::getSubject()->getObject($type, $id);

        //prepare the response object
        if ($object instanceof AAM_Core_Object) {
            $access   = $object->getOption();
            $metadata = array(
                'inherited'   => $object->getInherited(),
                'overwritten' => $object->isOverwritten()
            );
        } else {
            $access = $metadata = array();
        }

        return json_encode(array('access' => $access, 'meta' => $metadata));
    }
    
    /**
     * Save post properties
     * 
     * @return string
     * 
     * @access public
     */
    public function save() {
        if ($this->checkLimit()) {
            $subject = AAM_Backend_View::getSubject();
            
            $object = trim(AAM_Core_Request::post('object'));
            $id     = AAM_Core_Request::post('objectId', null);

            $param = AAM_Core_Request::post('param');
            $value = filter_var(
                    AAM_Core_Request::post('value'), FILTER_VALIDATE_BOOLEAN
            );
            
            //clear cache
            AAM_Core_Cache::clear();
            
            $result = $subject->save($param, $value, $object, $id);
        } else {
            $result = false;
            $error  = __('You reached your limitation.', AAM_KEY);
        }

        return json_encode(array(
                    'status' => ($result ? 'success' : 'failure'),
                    'error'  => (empty($error) ? '' : $error)
        ));
    }
    
    /**
     * Reset the object settings
     * 
     * @return string
     * 
     * @access public
     */
    public function reset() {
        $type = trim(AAM_Core_Request::post('type'));
        $id   = AAM_Core_Request::post('id', 0);

        $object = AAM_Backend_View::getSubject()->getObject($type, $id);
        if ($object instanceof AAM_Core_Object) {
            $result = $object->reset();
            //clear cache
            AAM_Core_Cache::clear();
        } else {
            $result = false;
        }
        
        return json_encode(array('status' => ($result ? 'success' : 'failure')));
    }

    /**
     * 
     * @global type $wpdb
     * @return type
     */
    protected function checkLimit() {
        global $wpdb;
        
        $limit = apply_filters('aam-post-limit', 0);
        
        if ($limit != -1) {
            //count number of posts that have access saved
            $query = "SELECT COUNT(*) as `total` FROM {$wpdb->postmeta} "
                   . "WHERE meta_key LIKE %s";
            
            $row = $wpdb->get_row($wpdb->prepare($query, 'aam_post_access_%'));
            $limit = ($row->total < 10 ? -1 : 0);
        }
        
        return ($limit == -1);
    }
    
    /**
     * @inheritdoc
     */
    public static function getAccessOption() {
        return 'feature.post.capability';
    }
    
    /**
     * @inheritdoc
     */
    public static function getTemplate() {
        return 'object/post.phtml';
    }
    
    /**
     * 
     * @return type
     */
    public function getCurrentPost() {
        $id = intval(AAM_Core_Request::post('oid'));
        
        if ($id) {
            $post = get_post($id);
        }
        
        return (isset($post) ? $post : null);
    }

    /**
     * Register Posts & Pages feature
     * 
     * @return void
     * 
     * @access public
     */
    public static function register() {
        $cap = AAM_Core_Config::get(self::getAccessOption(), 'administrator');
        
        AAM_Backend_Feature::registerFeature((object) array(
            'uid'        => 'post',
            'position'   => 20,
            'title'      => __('Posts & Pages', AAM_KEY),
            'capability' => $cap,
            'subjects'   => array(
                'AAM_Core_Subject_Role',
                'AAM_Core_Subject_User',
                'AAM_Core_Subject_Visitor'
            ),
            'view'       => __CLASS__
        ));
    }

}